/*
 * Copyright 2022 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef WPROBLEM_H
#define WPROBLEM_H

#include "heat.h"
#include "wObject.h"
#include <memory>
#include <iostream>
#include <vector>

using namespace tbox;

namespace heat
{

/**
 * @brief a class containing the problem parameters
 */

class HEAT_API Problem : public fwk::wSharedObject
{
public:
    std::shared_ptr<MshData> msh; ///< Mesh structure
#ifndef SWIG
    std::vector<std::shared_ptr<Medium>> media;   ///< Materials
    std::vector<std::shared_ptr<Dirichlet>> dBCs; ///< Dirichlet boundary conditions

    std::vector<std::shared_ptr<Source>> srcs;   ///< volumic sources
    std::vector<std::shared_ptr<Boundary>> bnds; ///< prescribed fluxes on the boundary

    std::shared_ptr<Periodic> pdic; ///< periodic BC
#endif

public:
    Problem(std::shared_ptr<MshData> _msh);
    virtual ~Problem();

    void add(std::shared_ptr<Medium> m) { media.push_back(m); }
    void add(std::shared_ptr<Dirichlet> d) { dBCs.push_back(d); }
    void add(std::shared_ptr<Source> s) { srcs.push_back(s); }
    void add(std::shared_ptr<Boundary> b) { bnds.push_back(b); }
    void add(std::shared_ptr<Periodic> p) { pdic = p; }

#ifndef SWIG
    void initElems();
    virtual void write(std::ostream &out) const override;
#endif
};

} // namespace heat

#endif //WPROBLEM_H
