
import numpy as np
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import axes3d
import matplotlib.tri as mtri

from katoptron.Juelich.UQ.halton import *
from katoptron.Juelich.UQ.copulas import *
import katoptron.utilities as u
from scipy.stats import lognorm


def main():

    N = 640
    n_rand = 2

    halton_points = np.zeros((n_rand, N))

    for i in range(0, N):
        halton_points[:, i] = halton(i + 1, n_rand)

    #fig = plt.figure()
    # plt.scatter(halton_points[0,:],halton_points[1,:])
    #ax = fig.gca()
    # ax.set_aspect('equal')

    np.random.seed(42)

    sigma = 0.15
    mu = np.log(345) + sigma**2  # mode = 345

    randomVariable_1 = np.random.lognormal(mu, sigma, N)
    randomVariable_2 = np.random.lognormal(mu, sigma, N)

    randomVariable_1_halton = lognorm.ppf(halton_points[0, :],
                                          sigma,
                                          scale=np.exp(mu))
    randomVariable_2_halton = lognorm.ppf(halton_points[1, :],
                                          sigma,
                                          scale=np.exp(mu))

    randomVariable_halton = lognormal(halton_points, [mu, mu], [sigma, sigma])

    #randomVariable_1_halton = lognorm.rvs(sigma,scale=np.exp(mu), size=N, random_state=42)
    #randomVariable_2_halton = lognorm.rvs(sigma,scale=np.exp(mu), size=N)

    #fig = plt.figure()
    # plt.scatter(randomVariable_1,randomVariable_2)
    #ax = fig.gca()
    # ax.set_aspect('equal')

    #fig = plt.figure()
    # plt.scatter(randomVariable_1_halton,randomVariable_2_halton)
    #ax = fig.gca()
    # ax.set_aspect('equal')

    fig = plt.figure()
    plt.scatter(randomVariable_1, randomVariable_2)
    plt.scatter(randomVariable_halton[0, :], randomVariable_halton[1, :])
    ax = fig.gca()
    ax.set_aspect('equal')
    plt.show()


if __name__ == "__main__":
    main()
