#ifndef KATOPTRON_ALGEBRAIC_H
#define KATOPTRON_ALGEBRAIC_H

#include "katoptron.h"
#include "Map.h"
#include "Domain.h"
#include "Loads.h"
#include "Graph.h"
#include "Matrices.h"
#include "Vectors.h"

#include "LinearSolver.h"

#include "EnsembleTraits.h"

#include <limits>

namespace katoptron
{

/**
 * @brief Class which is used to store Teuchos::RCP to the algebraic information of the problem:
 * <ul>
 *  <li> map: an object that stores all the maps,
 *  <li> graph: an object that stores all the graphs (the sparsity patterns of the matrices),
 *  <li> matrices: an object that stores all the matrices,
 *  <li> vectors: an object that stores all the vectors.
 * </ul>
 */
template <typename scalar>
class Algebraic
{
    typedef Map::local_ordinal_type local_ordinal_type;
    typedef Map::global_ordinal_type global_ordinal_type;

public:
    Teuchos::RCP<Map> map;
    Teuchos::RCP<Graph> graph;
    Teuchos::RCP<Matrices<scalar>> matrices;
    Teuchos::RCP<Vectors<scalar>> vectors;

    Algebraic(Teuchos::RCP<Map> map, Teuchos::RCP<Domain<scalar>> domain);

    Teuchos::RCP<Tpetra::Vector<scalar, local_ordinal_type, global_ordinal_type>> readVectorFromFile(std::string &name, size_t myRank);
};
}; // namespace katoptron

#endif //KATOPTRON_ALGEBRAIC_H
