#ifndef KATOPTRON_CONTACT_H
#define KATOPTRON_CONTACT_H

#include "katoptron.h"
#include "wGroup.h"
#include <vector>
#include <string>
#include <Eigen/Dense>

namespace katoptron
{

/**
 * @brief Class used to specify in Python the different contact interfaces.
 */
class KATOPTRON_API Contact : public tbox::Group
{
public:
    std::string Contact_name;
    std::vector<tbox::Element *> slave_elems;
    std::vector<tbox::Element *> master_elems;

    std::vector<int> slave_boundary_nodes;
    std::vector<int> master_boundary_nodes;

    std::vector<int> slave_dirichlet_nodes;
    std::vector<int> master_dirichlet_nodes;

    std::vector<int> initially_closed_nodes = {};

    bool initial_closed = true;
    bool standard_shape_functions = true;
    bool is_Signorini = false;
    bool no_Update = false;
    bool sticking = false;
    bool mesh_tying = false;

    int nb_Lagrange_multipliers = 1;

    Eigen::Vector3d slave_normal;
    Eigen::Vector3d master_normal;

    double dg;

    double scale = 1.;

    std::vector<double> dg_vector;

    Contact(Problem &pbl, int no, std::string _Contact_name, Eigen::Vector3d _slave_normal, double dg = 0.);
    Contact(Problem &pbl, std::string const &name, std::string _Contact_name, Eigen::Vector3d _slave_normal, double dg = 0.);
    Contact(Problem &pbl, int no, std::string _Contact_name, Eigen::Vector3d _slave_normal, std::vector<double> _dg);
    Contact(Problem &pbl, std::string const &name, std::string _Contact_name, Eigen::Vector3d _slave_normal, std::vector<double> _dg);

    void setMaster(katoptron::Problem &pbl, int no, Eigen::Vector3d _master_normal);
    void setMaster(katoptron::Problem &pbl, std::string const &name, Eigen::Vector3d _master_normal);

    void setSlaveBoundary(std::string const &file_name, std::string const &name);
    void setMasterBoundary(std::string const &file_name, std::string const &name);

    void setSlaveDirichlet(std::string const &file_name, std::string const &name);
    void setMasterDirichlet(std::string const &file_name, std::string const &name);

    void setInitialyClosed();
    void setInitialyOpen();
    void setNoUpdate();
    void setUpdate();
    void setMeshTying();
    void setSticking();
    void setNoSticking();
    void setStandardShapeFunctions();
    void setDualShapeFunctions();
    void setSignoriniContact();
    void setTwoBodyContact();
    void setInitiallyClosedNodes(std::vector<double> _initially_closed_nodes);
    void setScale(double a);

#ifndef SWIG
    virtual void write(std::ostream &out) const;
#endif
};

} // namespace katoptron

#endif //KATOPTRON_CONTACT_H
