#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2022 University of Liège
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.





import numpy as np
import os
import shutil


import params.src.write as write
import params.src.plot as plot
import params.src.indices as indices


def model(P):

    import mirrors.models.mirror_ss as mirror_model

    sol = mirror_model.runmodel(L = P[0], L_1 = P[1], H = P[2], H_1 = P[3], W = P[4], nL_1 = P[5], nL = P[6], nH = P[7], nH_1 = P[8], nW = P[9], T = P[10])

    return sol

def main():
    #--------------------------------------------------------
    normalisation = True
    #--------------------------------------------------------
    fname = 'mparams.log'
    #--------------------------------------------------------
    L = 250.0
    L_1 = 250.0
    H = 50
    H_1 = 30 
    W = 150
    #--------------------------------------------------------
    nL_1 = 11
    nL =  11
    nH =  6
    nH_1 = 6
    nW =  5 
    #--------------------------------------------------------
    T = 70
    #--------------------------------------------------------
    d = 2
    
    P = np.array([L, L_1, H, H_1, W, nL_1, nL, nH, nH_1, nW, T])
    P_indices = np.array([10, 3], dtype =np.int)

    M = np.zeros((d,), dtype =np.int)
    for j in range(0,d):
        M[j] = 3

    Dom_min = np.array([ 100., 2.])
    Dom_max = np.array([ 300., 10.])
    #--------------------------------------------------------
    N = np.zeros((d,), dtype =np.int)
    for j in range(0,d):
        N[j] = 3

    k = 9
    #--------------------------------------------------------
    from params.src.monomials import phi                      # monomials or legendre
    import params.src.gauss as tp                             # just gauss
    imported_sol = np.array([])
    imported_c = np.array([])
    imported_cs = np.array([])
    from mirrors.mparams_sols.s30x30 import import_sol        # not usefull if recomputed
    from mirrors.mparams_sols.s30x30_c_QR_pre_30x30 import import_c
    #imported_sol, d, M, Dom_min, Dom_max = import_sol()
    #imported_c, imported_cs, N, normalisation = import_c()
    solver = 'SVD' # norm_eq, Cholesky, QR, SVD, TSVD
    save = False
    #--------------------------------------------------------
    m1 = 40
    m2 = 40
    #---------------

    #d = 1
    #M = np.zeros((d,), dtype =np.int)
    #for j in range(0,d):
    #    M[j] = 30
    #N = np.zeros((d,), dtype =np.int)
    #for j in range(0,d):
    #    N[j] = 30
    #========================================================

    f = write.init(fname)

    X, w = tp.grid(M)  

    m_max = 1
    n_max = 1 
    for j in range(0,d):
        m_max = m_max*M[j] 
        n_max = n_max*N[j]

    i =      np.zeros((d,), dtype =np.int)
    i_zero = np.zeros((d,), dtype =np.int)
    i_one =  np.ones((d,), dtype =np.int)
    p = 0

    xi = np.zeros((d,))

    fig, ax, current_title = plot.init(d)

    if not os.path.exists('images'):
        os.mkdir('images')
    else:
        shutil.rmtree('images')
        os.mkdir('images')

    m_i = 0
    sol = np.zeros((m_max,1))

    Z = np.zeros((m_max,n_max))
    W = np.zeros((m_max,m_max))
    W_sr = np.zeros((m_max,m_max))    

    while np.all(i != -i_one):

        if normalisation == True:
            for j in range(0,d):
                xi[j] = X[i[j],j]
        elif normalisation == False:
            for j in range(0,d):
                xi[j] = X[i[j],j]
                xi[j] = Dom_min[j] + (Dom_max[j] - Dom_min[j])*(xi[j]+1.)/2.
                P[P_indices[j]] = xi[j]

        Z[m_i,:] = np.transpose(phi(xi,N,n_max,d))

        if normalisation == True:
            for j in range(0,d):
                xi[j] = Dom_min[j] + (Dom_max[j] - Dom_min[j])*(xi[j]+1.)/2.
                P[P_indices[j]] = xi[j]

        if imported_sol.size == 0:
            name = 'i_'
            for j in range(0,d-1):
                name = name + str(i[j]) + '_'
            name = name + str(i[d-1]) + '_'
            name = name + 'M_' 
            for j in range(0,d-1):
                name = name + str(M[j]) + '_'
            name = name + str(M[d-1]) 

            if not os.path.exists(name):
                os.mkdir(name)
            else:
                shutil.rmtree(name)
                os.mkdir(name)

            os.chdir(name)
            sol[m_i] = model(P)
            os.chdir('..')
            plot.scatter(xi,sol[m_i],d,fig,ax, True,m_i)

        elif imported_sol.size != 0:
            sol[m_i] = imported_sol[m_i]
            #plot.scatter(xi,sol[m_i],d,fig,ax, False,m_i)

        W[m_i,m_i] = 1
        for j in range(0,d):
            W[m_i,m_i] = W[m_i,m_i]*w[i[j],j]

        W_sr[m_i,m_i]  = np.sqrt(W[m_i,m_i])          
        p,i = indices.my_next(p,i,M,d)
        m_i = m_i + 1     



    #------------------

    x = np.linspace(-1.,1.,m1)
    y = np.linspace(-1.,1.,m2)
    X = np.zeros((m1,m2))
    Y = np.zeros((m1,m2))
    surrogate = np.zeros((m1,m2))
    X, Y = np.meshgrid(x, y)
    phi_tmp = np.zeros((m1,m2,n_max))

    #------------------
    for i in range(0,m1):
        for j in range(0,m2):
            if normalisation == False:
                X[i,j] = Dom_min[0] + (Dom_max[0] - Dom_min[0])*(X[i,j]+1.)/2.
                Y[i,j] = Dom_min[1] + (Dom_max[1] - Dom_min[1])*(Y[i,j]+1.)/2.
            xi = np.zeros((2,))
            xi[0] = X[i,j]
            xi[1] = Y[i,j]
            phi_tmp[i,j,:] = phi(xi,N,n_max,d)

    if normalisation == True:
        for i in range(0,m1):
            for j in range(0,m2):
                X[i,j] = Dom_min[0] + (Dom_max[0] - Dom_min[0])*(X[i,j]+1.)/2.
                Y[i,j] = Dom_min[1] + (Dom_max[1] - Dom_min[1])*(Y[i,j]+1.)/2.

    W_srZ = np.dot(W_sr,Z)
    U, s, V_t = np.linalg.svd(W_srZ, full_matrices=True)
    V = np.transpose(V_t)
    S = np.zeros((m_max,n_max))
    for j in range(0,n_max):
        S[j,j] = s[j]

    if solver == 'norm_eq': 
        ZWZ = np.dot(np.transpose(Z),np.dot(W,Z))
        rhs = np.dot(np.transpose(Z),np.dot(W,sol))
        s_hat = np.linalg.solve(ZWZ, rhs)
        for i in range(0,m1):
            for j in range(0,m2):
                surrogate[i,j] = np.dot(np.transpose(phi_tmp[i,j,:]),s_hat)

        #plot.surf(X, Y, surrogate, sol, fig, ax, current_title, True, True, 'Surrogate model','surrogate.png',d)

    elif solver == 'Cholesky': 
        ZWZ = np.dot(np.transpose(Z),np.dot(W,Z))
        rhs = np.dot(np.transpose(Z),np.dot(W,sol))
        R = np.linalg.cholesky(ZWZ)
        tmp =  np.linalg.solve(np.transpose(R), rhs)
        s_hat = np.linalg.solve(R, tmp)
        for i in range(0,m1):
            for j in range(0,m2):
                surrogate[i,j] = np.dot(np.transpose(phi_tmp[i,j,:]),s_hat)

        #plot.surf(X, Y, surrogate, sol, fig, ax, current_title, True, True, 'Surrogate model','surrogate.png',d)

    elif solver == 'QR': 
        W_srZ = np.dot(W_sr,Z)
        Q, R = np.linalg.qr(W_srZ)
        rhs = np.dot(np.transpose(Q),np.dot(W_sr,sol))
        s_hat = np.linalg.solve(R, rhs)
        for i in range(0,m1):
            for j in range(0,m2):
                surrogate[i,j] = np.dot(np.transpose(phi_tmp[i,j,:]),s_hat)

        #plot.surf(X, Y, surrogate, sol, fig, ax, current_title, True, True, 'Surrogate model','surrogate.png',d)



    elif solver == 'SVD': 
        #W_srZ = np.dot(W_sr,Z)
        #U, s, V_t = np.linalg.svd(W_srZ, full_matrices=False)


        V = np.transpose(V_t)

        rhs = np.dot(np.transpose(U),np.dot(W_sr,sol))
        s_hat = np.dot(V,np.linalg.solve(S, rhs))

        for i in range(0,m1):
            for j in range(0,m2):
                surrogate[i,j] = np.dot(np.transpose(phi_tmp[i,j,:]),s_hat)

        #plot.surf(X, Y, surrogate, sol, fig, ax, current_title, True, True, 'Surrogate model','surrogate.png',d)

    elif solver == 'TSVD': 
        #W_srZ = np.dot(W_sr,Z)
        #U, s, V_t = np.linalg.svd(W_srZ, full_matrices=True)
        S = np.zeros((m_max,n_max))
        for j in range(0,n_max):
            S[j,j] = s[j]

        s_hat = np.zeros((n_max,))

        V = np.transpose(V_t)
        for jj in range(0,min(n_max,k)):
            s_hat = s_hat + np.dot(np.transpose(U[:,jj]),np.dot(W_sr,sol))*V[:,jj]/S[jj,jj] 

            for i in range(0,m1):
                for j in range(0,m2):
                    surrogate[i,j] = np.dot(np.transpose(phi_tmp[i,j,:]),s_hat)

            plot.surf(X, Y, surrogate, sol, fig, ax, current_title, False, save, 'Surrogate model with $k = ' + str(jj+1) +'$','surrogate' + str(jj+1) + '.png',d)

        #plot.surf(X, Y, surrogate, sol, fig, ax, current_title, True, True, 'Surrogate model with $k = ' + str(jj+1) +'$','surrogate' + str(jj+1) + '.png',d)


    #----------------------
    #s_1 = np.array([1.56657114908, 1.11460642267, 0.643552021341, 0.352673584151, 0.157153535614, 0.0769603917451, 0.0265674123834, 0.0120471071617, 0.00282921208345, 0.00120678298737])
    s_1 = np.array([1.61820129502, 1.25403803215, 0.852380849704, 0.541995080368, 0.320759977206, 0.188184793223, 0.102611498787, 0.0571855117691, 0.0290259910435, 0.0155199291833, 0.00734487330744, 0.00378954767668, 0.00166939903549, 0.000834429285046, 0.000340818529243, 0.000165534152138, 6.22971490317e-05, 2.94723359133e-05, 1.01274211329e-05, 4.67622340761e-06, 1.44805698661e-06, 6.53687563397e-07, 1.78871573953e-07, 7.90594553641e-08, 1.85213020878e-08, 8.02554270891e-09, 1.51844326981e-09, 6.45791166698e-10, 8.54979264514e-11, 3.57263540233e-11])
    s_2 = np.zeros((900,))
    kk = 0
    for ii in range(0,30):
        for jj in range(0,30):
            s_2[kk] = s_1[ii]*s_1[jj]
            kk = kk + 1 
    #plot.SVD(900,-np.sort(-s_2))
    #plot.SVD(n_max,s)
    c = np.dot(V_t,s_hat)
    #fig, ax = plot.plot2D(range(1,n_max+1),c,'$i [-]$','$c_i [-]$',"c_i.png")
    if imported_c.size != 0:
        #plot.add2D(range(1,n_max+1),imported_c,'ro',"c_i2.png")
        tmp = imported_c
        for jj in range(0,n_max):
            tmp[jj] = tmp[jj] - c[jj]
        #fig, ax = plot.plot2D(range(1,n_max+1),tmp,'$i [-]$','$dc_i [-]$',"dc_i.png")

    cs = np.dot(S,c)
    #fig, ax = plot.plot2D(range(1,n_max+1),cs,'$i [-]$','$c_i \sigma_i [-]$',"c_i_s_i.png")
    if imported_cs.size != 0:
        #plot.add2D(range(1,n_max+1),imported_cs,'ro',"cs_i2.png")
        tmp = imported_cs
        for jj in range(0,n_max):
            tmp[jj] = tmp[jj] - cs[jj]
        #fig, ax = plot.plot2D(range(1,n_max+1),tmp,'$i [-]$','$dc_i \sigma_i [-]$',"dc_i_s_i.png")

    ZWZ = np.dot(np.transpose(Z),np.dot(W,Z))
    write.col(f, np.diag(ZWZ), n_max)

    write.col(f, V[:,1], n_max)
    print(max(np.diag(ZWZ)))
    print(max(s))
    #----------------------

    write.sol(f, sol, m_max)
    write.c(f, c, n_max)
    write.c(f, s, n_max)
    write.cs(f, cs, n_max)
    write.finish(f)
    plot.finish()
    

if __name__ == "__main__":
    main()
