/*
 * Copyright 2022 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "wProblem.h"
#include "wMshData.h"
#include "wMedium.h"
#include "wuDirichlet.h"
#include "wuNeumann.h"
#include "wTDirichlet.h"
#include "wTNeumann.h"
#include "wTSource.h"
#include "wElement.h"

#include "wTag.h"
using namespace mirrors;

Problem::Problem(std::shared_ptr<MshData> _msh, std::string _problem_type, double _T_ref) : msh(_msh), problem_type(_problem_type), T_ref(_T_ref)
{
}

/**
 * @brief Initialize the elements precomputed values
 */
void Problem::initElems()
{
    // Update volume Jacobian
    for (auto vol : media)
        for (auto e : vol->tag->elems)
            e->initValues(true);
    // Update volume Jacobian (volumic sources)
    for (auto vol : Tsrcs)
        for (auto e : vol->tag->elems)
            e->initValues(true);
    // Update surface Jacobian (Neumann B.C.)
    for (auto surf : nuBCs)
        for (auto e : surf->tag->elems)
            e->initValues(false);
    for (auto surf : nTBCs)
        for (auto e : surf->tag->elems)
            e->initValues(false);
}

void Problem::write(std::ostream &out) const
{
    out << "mirrors::Problem:\n";
    out << problem_type << "\n";
    out << " + mesh:\n";
    out << *msh; //wMshData.h
    out << " + media:\n";
    for (auto m : media)
        out << *m << '\n';
    out << " + mechanical Dirichlet boundary conditions:\n";
    for (auto b : duBCs)
        out << *b << '\n';
    out << " + thermal Dirichlet boundary conditions:\n";
    for (auto b : dTBCs)
        out << *b << '\n';
    out << " + volumic heat sources:\n";
    for (auto b : Tsrcs)
        out << *b << '\n';
    ;
    out << " + mechanical von Neumann boundary conditions:\n";
    for (auto b : nuBCs)
        out << *b << '\n';
    out << " + thermal von Neumann boundary conditions:\n";
    for (auto b : nTBCs)
        out << *b << '\n';
}
