/*
 * Copyright 2022 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef WSOLVER_H
#define WSOLVER_H

#include "mirrors.h"
#include "wObject.h"
#include "wTimers.h"
#include <iostream>
#include <memory>

namespace mirrors
{

/**
 * @brief a class containing the problem parameters
 */

class MIRRORS_API Solver : public fwk::wObject
{
public:
#ifndef SWIG
    Problem &pbl;
#endif
    std::shared_ptr<tbox::LinearSolver> linsol; ///< linear solver
    fwk::Timers tms;                            ///< timers
    int nthreads;                               ///< nb of threads for the assembly

public:
    Solver(Problem &pbl, std::shared_ptr<tbox::LinearSolver> _linsol, int nthreads = 1);

    void start(tbox::MshExport *mshWriter);
#ifndef SWIG
    virtual void write(std::ostream &out) const override;
#endif
};

} // namespace mirrors

#endif //WSOLVER_H
