#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2022 University of Liège
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import numpy as np
import os
import shutil
import params.src.write as write
import params.src.indices as indices

from fwk.wutils import parseargs
args = parseargs()
if not args.nogui:
    import params.src.plot as plot

def model(P):

    import mirrors.models.mirror_ss as mirror_model

    sol = mirror_model.runmodel(L = P[0], 
                                L_1 = P[1], 
                                H = P[2], 
                                H_1 = P[3], 
                                W = P[4], 
                                nL_1 = P[5], 
                                nL = P[6], 
                                nH = P[7], 
                                nH_1 = P[8],
                                nW = P[9], 
                                T = P[10])

    return sol

def main():
    #--------------------------------------------------------
    normalisation = True
    #--------------------------------------------------------
    fname = 'mparams.log'
    #--------------------------------------------------------
    L = 250.0
    L_1 = 250.0
    H = 50
    H_1 = 30 
    W = 150
    #--------------------------------------------------------
    nL_1 = 11
    nL =  11
    nH =  6
    nH_1 = 6
    nW =  5 
    #--------------------------------------------------------
    T = 70
    #--------------------------------------------------------
    d = 2
    
    P = np.array([L, L_1, H, H_1, W, nL_1, nL, nH, nH_1, nW, T])
    P_indices = np.array([10, 3], dtype =np.int)

    M = np.zeros((d,), dtype =np.int)
    for j in range(0,d):
        M[j] = 10

    Dom_min = np.array([ 100., 2.])
    Dom_max = np.array([ 300., 10.])
    #--------------------------------------------------------
    N = np.zeros((d,), dtype =np.int)
    for j in range(0,d):
        N[j] = 10

    k = 100
    #--------------------------------------------------------
    from params.src.monomials import phi                      # monomials or legendre
    import params.src.gauss as tp                             # just gauss
    imported_sol = np.array([])
    from mirrors.mparams_sols.s30x30 import import_sol        # not usefull if recomputed
    imported_sol, d, M, Dom_min, Dom_max = import_sol()
    solver = 'TSVD' # Cholesky, QR, SVD, TSVD
    #--------------------------------------------------------
    m1 = 40
    m2 = 40
    
    #========================================================

    f = write.init(fname)

    X, w = tp.grid(M)  

    m_max = 1
    n_max = 1 
    for j in range(0,d):
        m_max = m_max*M[j] 
        n_max = n_max*N[j]

    i =      np.zeros((d,), dtype =np.int)
    i_zero = np.zeros((d,), dtype =np.int)
    i_one =  np.ones((d,), dtype =np.int)
    p = 0

    xi = np.zeros((d,))

    if not args.nogui:
        fig, ax, current_title = plot.init(d)

        if not os.path.exists('images'):
            os.mkdir('images')
        else:
            shutil.rmtree('images')
            os.mkdir('images')

    m_i = 0
    sol = np.zeros((m_max,1))

    Z = np.zeros((m_max,n_max))
    W = np.zeros((m_max,m_max))
    W_sr = np.zeros((m_max,m_max))    

    while np.all(i != -i_one):

        if normalisation == True:
            for j in range(0,d):
                xi[j] = X[i[j],j]
        elif normalisation == False:
            for j in range(0,d):
                xi[j] = X[i[j],j]
                xi[j] = Dom_min[j] + (Dom_max[j] - Dom_min[j])*(xi[j]+1.)/2.
                P[P_indices[j]] = xi[j]

        Z[m_i,:] = np.transpose(phi(xi,N,n_max,d))

        if normalisation == True:
            for j in range(0,d):
                xi[j] = Dom_min[j] + (Dom_max[j] - Dom_min[j])*(xi[j]+1.)/2.
                P[P_indices[j]] = xi[j]

        if imported_sol.size == 0:
            name = 'i_'
            for j in range(0,d-1):
                name = name + str(i[j]) + '_'
            name = name + str(i[d-1]) + '_'
            name = name + 'M_' 
            for j in range(0,d-1):
                name = name + str(M[j]) + '_'
            name = name + str(M[d-1]) 

            if not os.path.exists(name):
                os.mkdir(name)
            else:
                shutil.rmtree(name)
                os.mkdir(name)

            os.chdir(name)
            sol[m_i] = model(P)
            os.chdir('..')
            if not args.nogui:
                plot.scatter(xi,sol[m_i],d,fig,ax, True,m_i)

        elif imported_sol.size != 0:
            sol[m_i] = imported_sol[m_i]
            if not args.nogui:
                plot.scatter(xi,sol[m_i],d,fig,ax, False,m_i)

        W[m_i,m_i] = 1
        for j in range(0,d):
            W[m_i,m_i] = W[m_i,m_i]*w[i[j],j]

        W_sr[m_i,m_i]  = np.sqrt(W[m_i,m_i])          
        p,i = indices.my_next(p,i,M,d)
        m_i = m_i + 1     

    W_srZ = np.dot(W_sr,Z)
    U, s, V_t = np.linalg.svd(W_srZ, full_matrices=True)
    S = np.zeros((m_max,n_max))
    for j in range(0,n_max):
        S[j,j] = s[j]

    s_hat = np.zeros((n_max,))

    V = np.transpose(V_t)

    #------------------

    x = np.linspace(-1.,1.,m1)
    y = np.linspace(-1.,1.,m2)
    X = np.zeros((m1,m2))
    Y = np.zeros((m1,m2))
    surrogate = np.zeros((m1,m2))
    X, Y = np.meshgrid(x, y)
    phi_tmp = np.zeros((m1,m2,n_max))

    if not args.nogui:
        plot.SVD(n_max,s)

    write.sol(f, sol, m_max)

    write.finish(f)

    if not args.nogui:
        plot.finish()
    

if __name__ == "__main__":
    main()
