#!/usr/bin/env python3
# -*- coding: utf-8; -*-

# Copyright 2022 University of Liège
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.



import numpy as np

def main():

    from params.arnst_ponthot.src import gamma

    import matplotlib.pyplot as plt

    xi_tp = np.array([[-2.02018,-2.02018],
         [-2.02018,-0.958572],
         [-2.02018,0],
         [-2.02018,0.958572],
         [-2.02018,2.02018],
         [-0.958572,-2.02018],
         [-0.958572,-0.958572],
         [-0.958572,0],
         [-0.958572,0.958572],
         [-0.958572,2.02018],
         [0,-2.02018],
         [0,-0.958572],
         [0,0],
         [0,0.958572],
         [0,2.02018],
         [0.958572,-2.02018],
         [0.958572,-0.958572],
         [0.958572,0],
         [0.958572,0.958572],
         [0.958572,2.02018],
         [2.02018,-2.02018],
         [2.02018,-0.958572],
         [2.02018,0],
         [2.02018,0.958572],
         [2.02018,2.02018]], dtype=float)
    m_H = 1495.
    sigmaS_H = 1390.
   
    m_S = 396.
    sigmaS_S = 660.

    rho = -0.223

    g_H = np.linspace(1200,1800, 100)
    g_S = np.linspace(250,550, 100)
    loc = 0. 
    
    PDF = gamma.rho_gamma_gamma(g_H,m_H,sigmaS_H,g_S,m_S,sigmaS_S,rho)

    G_H, G_S = np.meshgrid(g_H, g_S)

    fig, ax = plt.subplots()
    plt.contour(G_H, G_S, PDF)

    for i in range(0,25):
        tmp = gamma.f_mapping(xi_tp[i,1],m_S,sigmaS_S,xi_tp[i,0],m_H,sigmaS_H,rho)
        plt.scatter(tmp[1],tmp[0])
    ax.grid(True)

    plt.show()
if __name__ == "__main__":
    main()
