#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2022 University of Liège
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.




import numpy as np
import time

def init(fname):
    date = time.strftime("%d/%m/%Y")
    hour = time.strftime("%H:%M:%S")

    f = open(fname,'w')
    f.write('Parametric loop\n')
    f.write('Simulations started at: ' + date + ' ' + hour + '.\n')
    f.write('====================================================================================\n')

    return f

def sol(f, sol, m_max):
    f.write('sol = np.array([')
    for i in range(0,m_max-1):
        f.write(str(sol.item(i))+', ')
    f.write(str(sol.item(m_max-1))+'])\n')

def c(f, c, n_max):
    f.write('c = np.array([')
    for i in range(0,n_max-1):
        f.write(str(c.item(i))+', ')
    f.write(str(c.item(n_max-1))+'])\n')

def cs(f, cs, n_max):
    f.write('cs = np.array([')
    for i in range(0,n_max-1):
        f.write(str(cs.item(i))+', ')
    f.write(str(cs.item(n_max-1))+'])\n')

def col(f, col, n_max):
    f.write('-----------')
    for i in range(0,n_max):
        f.write(str(col.item(i))+', \n')

def w_list(f, list1, name):
    f.write(name+':\n')
    for k,v in list(list1.items()):
        line = '{}: {}'.format(k,v) 
        data = line.split()
        for word in data:
            f.writelines('%20s ' % word)
        f.write('\n')
    f.write('------------------------------------------------------------------------------------\n')

def w_vector(f, cv, name):
    f.write(name+':\n')
    for i in cv:
        f.write(str(i)+' ')
    f.write('\n------------------------------------------------------------------------------------\n')

def w_MPI_info(f, name, siz):
    f.write('Name: ' + name +'\n')
    f.write('Size: ' + str(siz) +'\n')
    f.write('------------------------------------------------------------------------------------\n')

def w_timers(f, timers):
    f.write(str(timers))

def finish(f):
    date = time.strftime("%d/%m/%Y")
    hour = time.strftime("%H:%M:%S")

    f.write('====================================================================================\n')
    f.write('Simulations finished at: ' + date + ' ' + hour + '.\n')

    f.close()
